<?php
/* 
 *******************************************
 plugin JUpload for Coppermine Photo Gallery
 *******************************************

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 ********************************************
 $Revision: 185 $
 $Author: etienne_sf $
 $Date: 2008-03-12 20:26:16 +0100 (mer., 12 mars 2008) $
 ********************************************
 *
 * Allows easy upload to the gallery, through a java applet. 
 * 
 * Up to date version of this script can be retrieved with the full JUpload package, here:
 * 
 * http://etienne.sf.free.fr/wiki
 * 
 * Directly here:
 * http://etienne.sf.free.fr/wiki/doku.php?id=jupload_coppermine_download_gb
 * 
 * Support is available on this forum:
 * http://coppermine-gallery.net/forum/index.php?topic=43432
 * 
 * The applet is published on sourceforge:
 * http://jupload.sourceforge.net
 * 
 */

/*
 * This include is meant to add additional functions, for script that need to manage pictures.
 * 
 * It is based on the folowing facts:
 * 1) A lot of scripts have specific internal functions to manage the incoming file. 
 *   At least these scripts: db_input, upload, jupload, addpic, xp_publish.
 * 2) Each are slightly different, which means: it's not possible to directly reuse them in another script
 * 3) As these scripts also do input (mix between code that produce output and code that should be included), it's
 *   not possible to include them in another one to reuse code.
 * 4) If a bug exists (especially if it involve security), it's possible that several correction have to be done.
 * 5) Changing anything in the picture management system may become difficult, because many functions can have to be
 * modified.
 * 
 * 
 * This j_picmgmt.inc.php script contains two _new_ functions, taken from picmgmt.inc.php:
 * a) jupload_standard_add_picture. Used in all other script.
 *   It contains :
 *    - Filetype checks. The xp_publish script also tests this. But, for sure, there must be differences between
 *     these two scripts. 
 *    - Quota management: to insure that the user respects his quota.
 *    - Big, thumb and normal picture calculation (by calls to jupload_standard_resize_image)
 *    - Insert into the database  
 *   But, as I understand it, it lacks :
 *    - Right management. This function assumes that the caller checks if the current user may upload into this
 *     album.
 *    - Moving file to the right directory
 *    - Characters filename control. It also assumes that the file contains only valid characters. For instance, 
 *     filename containing '/', '\' or '.' characters should be carefully managed (also done in xp_publish). Also, 
 *     from
 *     my test, coppermine seems to badly managed spaces, and non-ascii characters (like accents).
 *   
 * b) jupload_standard_resize_image. No trouble: it allows construction of target file, thumn and normal pictures, of the good size, 
 * according to the current gallery configuration. This function should only be called by the add_picture function.
 */

if (!defined('IN_COPPERMINE')) { die('Not in Coppermine...');}
if (!defined('JUPLOAD_PHP')) { die('Not in Jupload...');}



/**
 * $inStrammModPack indicates whether the Stramm's modpack is installed in the current system, or not.
 */ 
$inStrammModPack = false;

echo("[WARNING] \$inStrammModPack forced to false. Local clone of Stramm function will not be used.\r\n");


/*
 * Let's include the relevant j_picmgmt file.
 * 
 * If the runkit_function_redefine function exists, then these include are useless, as jupload will use 
 * the available Coppermine functions
 */
$functionName = 'runkit_function_redefine';
//$useRunkitFunctionRedefine = function_exists($functionName) && is_callable($functionName);
//logDebug("Checked possible access to function $functionName. Result: " . 
//	($useRunkitFunctionRedefine ? 'true' : 'false')
//	);
$useRunkitFunctionRedefine = false;
echo("[WARNING] \$useRunkitFunctionRedefine forced to false. Won't use available Coppermine functions (only JUpload clone of them).\r\n");

// Let's define which add_picture() and resize_image() jupload will use. To do that, we include the relevant include file.
if ($useRunkitFunctionRedefine) {
	include("plugins/jupload/include/j_picmgmt_current.inc.php");
} else if ($inStrammModPack) {
	include("plugins/jupload/include/j_picmgmt_stramm.inc.php");
} else {
	include("plugins/jupload/include/j_picmgmt_standard.inc.php");
}

logDebug('End of j_picmgmt.inc.php');